from FatxFast.gui.containerwidget import ContainerWidget
from FatxFast.gui.textwidget import TextWidget
from FatxFast.gui.boxwidget import BoxWidget
from FatxFast.gui.menuwidget import MenuWidget
from FatxFast.constants import *
import datetime
import time
import pygame

class Frame(ContainerWidget):

    def __init__(self, game, **kwargs):
        self.game = game
        super(Frame, self).__init__(**kwargs)

    def fetch_template(self, name):
        return self.game.manager.sceneparser.templates[name].copy()

    def create_text(self, new_attr, template="gametext"):
        attr = self.fetch_template(template)
        a = self.game.manager.sceneparser.override_attributes(attr, new_attr)
        text = TextWidget(**a)
        self.add_widget(text.name, text)
        return text

class GameBox(Frame):
    
    def __init__(self, game, **kwargs):
        super(GameBox, self).__init__(game, **kwargs)
        self.game.add_widget("game", self)
        self._hud = GlobalHud(game,**self.fetch_template("gamebar"))
        self.add_widget("global-hud", self._hud)
        self._player_count = 0

    def add_player(self, player, camera):
        attr = self.fetch_template("gamebox")
        attr["container"] = self
        widget = GameFrame(self.game, player, camera, **attr)
        widget.name = "hud-"+str(self._player_count)
        self.add_widget(widget.name, widget) 
        self._player_count += 1
        self._fit_gameframes()
        if self._player_count > 1:
            self._add_separator(self._player_count-1)

    def _add_separator(self, index):
        attr = self.fetch_template("separator")
        attr["container"] = self
        frame = self.get_widget("hud-"+str(index))
        attr["x"] = frame.rect.width*index
        attr["height"] = frame.rect.height
        attr["width"] = TILEWIDTH
        widget = BoxWidget(**attr)
        widget.name = "separator-"+str(index)
        self.add_widget(widget.name, widget)

    def _fit_gameframes(self):
        for index in xrange(0, self._player_count):
            frame = self.get_widget("hud-"+str(index))
            frame.rect.width = \
                (SCREENSIZE[0]-((self._player_count-1)*TILEWIDTH)) \
                    /self._player_count
            frame.rect.x = index*(frame.rect.width+TILEWIDTH)
    
    def remove_players(self):
        for index in xrange(0, self._player_count):
            self.delete_widget("hud-"+str(index))
            self.delete_widget("separator-"+str(index))
        self._player_count = 0

class GlobalHud(Frame):
    # Unified objects like time &  level
    def __init__(self, game, **kwargs):
        super(GlobalHud, self).__init__(game, **kwargs)
        self._cache_time = 0
        self._cache_level = 0
        self.create_text(
            {"name": "clock-icon",
                "x": 100,
                "y": 5,
                "size": 24,
                "container": self,
                "text": u"\ue803",
                "font": "fontello.ttf"})
        self._time_widget = self.create_text(
            {
                "font": "lavoir.ttf",
                "name":"time",
                "x":140,
                "y": 5,
                "text":"00:00:00",
                "container":self
            })
        self._level_widget = self.create_text({"name":"level",
            "x":800,
            "font": "lavoir.ttf",
            "y": 5,
            "text":"LEVEL: 0",
            "container":self})

    def update(self, dt):
        super(GlobalHud, self).update(dt)
        if self._cache_time != self.game.time/1000 and self.game.time > -1:
            self._cache_time = self.game.time/1000
            time_text = datetime.timedelta(seconds=self._cache_time)
            time_text = time.strftime("%H:%M:%S", time.strptime(str(time_text), 
                "%H:%M:%S"))
            self._time_widget.text = "%s" % time_text
            
        if self._cache_level != self.game.levelsystem.level:
            self._cache_level = self.game.levelsystem.level
            self._level_widget.text= "LEVEL: {0}".format(
                self.game.levelsystem.level)
        

class GameFrame(Frame):
    # Frame which is bound to a player 
    def __init__(self, game, player, camera, **kwargs):
        super(GameFrame, self).__init__(game, **kwargs)
        self.add_widget("buff-frame-"+str(id(player)), 
            BuffFrame(game, player, **self.fetch_template("buffbar"))) 
        self.add_widget(
            "attribute-frame-"+str(id(player)), 
            AttributesFrame(game, player, 
                **self.fetch_template("attributebar")))
        self.tilemap = self.game.tilemap
        self.camera = camera
        self._temp_surf = pygame.Surface(self.camera.rect.size)

    def _set_camera_zoom(self):
        if self._temp_surf.get_size() != self.camera.rect.size:
            self._temp_surf = pygame.Surface(self.camera.rect.size)

    def draw(self, surface): 
        if self.camera.rect.size != self.rect.size:
            self._set_camera_zoom()
            self.tilemap.draw(self._temp_surf, self.camera)
            surf = pygame.transform.scale(self._temp_surf, self.rect.size)
            surface.blit(surf, (0,0))
        else:
            surf_rect = surface.get_rect()
            sub = surface.subsurface(self.rect.clip(surf_rect))
            self.tilemap.draw(sub, self.camera)
        super(GameFrame, self).draw(surface)

class BuffFrame(Frame):
    # Part of GameFrame
    def __init__(self, game, player, **kwargs):
        super(BuffFrame, self).__init__(game, **kwargs)
        self.player = player
        self._last_update = 0
        self._update_interval = 0.33
        self._buff_container = MenuWidget(**kwargs)
        self.add_widget("buff-container", self._buff_container)
        # Hash of buff pointers to buff widgets

    def _create_buff(self, buff):
        attr = self.fetch_template("buffwidget")
        attr["name"] = buff
        attr["container"] = self
        attr["colorkey"] = buff.image.get_colorkey()
        buffwidget = BoxWidget(**attr)
        buffwidget.image = buff.image
        self._buff_container.add_widget(buff, buffwidget)
        """creates the (timer)text for the buffwidget"""
        attr = self.fetch_template("bufftext")
        attr["container"] = buffwidget
        bufftext = TextWidget(**attr)
        buffwidget.add_widget(attr["name"]+"-"+str(id(buff)), bufftext)

    def _remove_expired_buffs(self):
        for buff in self._buff_container.widgets.keys():
            if buff not in self.player.buffs:
                del self._buff_container.widgets[buff]
                self._buff_container.fix_children_y_pos()

    def _add_new_buff(self, buff): 
        if buff not in self._buff_container.widgets:
            self._create_buff(buff) 

    def update(self, dt):
        if time.time()-self._last_update > self._update_interval:
            for buff in self.player.buffs:
                self._add_new_buff(buff)
                self._buff_container.widgets[buff].\
                    widgets["bufftext-"+str(id(buff))].text \
                    = str(buff.duration/1000)
            self._remove_expired_buffs()
            self._last_update = time.time()

class AttributesFrame(Frame):
    # Part of GameFrame
    def __init__(self, game, player, **kwargs):
        super(AttributesFrame, self).__init__(game, **kwargs)  
        self._player = player
        self._cache_points = self._player.points
        self._cache_maps = self._player.maps_left
        self.create_text(
            {"name": "points-icon",
                "x": 65,
                "y": 56,
                "size": 26,
                "container": self,
                "text": u"\ue800",
                "font": "fontello.ttf"})
        self._points_widget = self.create_text(
            {"name": "points",
                "font": "lavoir.ttf",
                "x": 100,
                "y": 60,
                "container": self,
                "text": "POINTS: {0}".format(self._player.points)})
        self.create_text(
            {"name": "maps-left-icon",
                "color": "gold3",
                "x": 65,
                "y": 90,
                "size": 26,
                "container": self,
                "text": u"\ue801",
                "font": "fontello.ttf"})
        self._maps_left_widget = self.create_text(
            {"name": "maps-left"+str(id(player)),
                "color": "gold3",
                "font": "lavoir.ttf",
                "x": 100,
                "y": 90,
                "container": self,
                "text": "{0}".format(self._player.maps_left)})

    def update(self, dt):
        if self._cache_points != self._player.points:
            self._cache_points = self._player.points
            self._points_widget.text = "POINTS: {0}".format(self._cache_points)
        if self._cache_maps != self._player.maps_left:
            self._cache_maps = self._player.maps_left
            self._maps_left_widget.text = "{0}".format(self._cache_maps)
